require 'pathname'
require 'logger'
require 'thread'
require 'json'

require 'slog'
require 'daybreak'
require 'deep_merge'
require 'sinatra/base'

require_relative 'helpers'
require_relative 'metadata'


module DBum

  # Web frontend for DBum, backed by Sinatra.
  class App < Sinatra::Application
    include DBum::Helpers

    def self.config! options
      @@logger = Slog::Logger.new
      @@db = Daybreak::DB.new options.database

      self.set :environment, options.environment
      self.set :port, options.port
      self.set :bind, options.bind
      self.set :raise_errors, options.debug?
      self.set :dump_errors, options.debug?
      self.set :show_exceptions, options.debug?
      self.set :logging, ::Logger::DEBUG if options.debug?
    end

    def self.close!
      @@db.close
    end


    get '/' do
      pretty_json keys
    end

    get '/:key' do
      key = params[:key]
      val = lookup key
      return status(404) if val.nil?
      pretty_json val
    end

    post '/:key' do
      key = params[:key]
      val = lookup key
      return status(404) unless val.nil?
      data = request_body
      update key, data
      pretty_json data
    end

    put '/:key' do
      key = params[:key]
      val = lookup key
      return status(404) if val.nil?
      data = request_body
      update key, data
      pretty_json data
    end

    patch '/:key' do
      key = params[:key]
      val = lookup key
      return status(404) if val.nil?
      val.deep_merge! request_body
      update key, val
      pretty_json val
    end

    delete '/:key' do
      key = params[:key]
      val = lookup key
      return status(404) if val.nil?
      remove key
      pretty_json val
    end


  private

    def request_body
      request.body.rewind
      JSON.parse request.body.read
    end

    def pretty_json o
      content_type :json
      JSON.pretty_generate o
    end

    def keys ; @@db.keys end

    def lookup key ; @@db[key] end

    def update key, val
      @@db.lock do
        @@db[key] = val
      end
      @@db.flush
    end

    def remove key
      @@db.lock do
        @@db.delete key
      end
      @@db.flush
    end

  end
end